# -*- coding: utf-8 -*-
"""Constants for pyps4-homeassistant."""
MAJOR_VERSION = 0
MINOR_VERSION = 5
PATCH_VERSION = 0
REQUIRED_PYTHON_VER = (3, 5, 3)

__short_version__ = '{}.{}'.format(MAJOR_VERSION, MINOR_VERSION)
__version__ = '{}.{}'.format(__short_version__, PATCH_VERSION)

__name__ = 'pyps4_homeassistant'
